classicHeals = {}; -- main scope to prevent scoping interference
local ch = classicHeals;

local eventFrame = CreateFrame("Frame");
eventFrame:RegisterEvent("ADDON_LOADED");
eventFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

local charName = UnitName("player");
local initFinished = false;

local partyMembers = nil;
local partyFrames = {};

local OPT_SHOWOVERHEAL = "OPT_SHOWOVERHEAL";

SLASH_CLASSICHEALS1 = "/classicheals";
local CMD_OVERHEAL = "overheal";

SlashCmdList["CLASSICHEALS"] = function(msg)
	if msg ~= nil then
		if msg == CMD_OVERHEAL then
			CLASSICHEALS_OPTIONS[OPT_SHOWOVERHEAL] = not CLASSICHEALS_OPTIONS[OPT_SHOWOVERHEAL];
			ch:Println("overheal: " .. tostring(CLASSICHEALS_OPTIONS[OPT_SHOWOVERHEAL]));
			return;
		end
	end
	
	ch:Println(ch:GetText(ch.LKEY_COMMANDS));
	ch:Println(ch:GetText(ch.LKEY_COMMANDS_OVERHEAL));
end

eventFrame:SetScript("OnEvent", function(self, event)
	if event ~=nil then
		if event == "ADDON_LOADED" and not initFinished then
			ch.Init();
		elseif event == "GROUP_ROSTER_UPDATE" then
			partyMembers = nil;
		elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
			self:OnEvent(event, CombatLogGetCurrentEventInfo());
		end
	end
end);

function eventFrame:OnEvent(event, ...)
	local timestamp, subevent, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = ...;

	local partyMemberId = ch:GetAffectedPartyMemberId(destName);
	if partyMemberId ~= nil then
		if sourceName == charName and subevent == "SPELL_HEAL" then
			spellId, spellName, spellSchool, amount, overhealing, absorbed, critical = select(12, ...);
			ch:DisplayHeal(partyFrames[partyMemberId], _G["PartyMemberFrame" .. partyMemberId], amount, overhealing);
		end
	end
end

function ch:Init()
	if CLASSICHEALS_OPTIONS == nil then
		CLASSICHEALS_OPTIONS = {};
		CLASSICHEALS_OPTIONS[OPT_SHOWOVERHEAL] = false;
	end

	initFinished = true;
	ch:Println(ch:GetText(ch.LKEY_ADDONLOADED));
end

function ch:DisplayHeal(frame, parentFrame, amount, overhealing)	
	local message;
	if overhealing > 0 then
		if CLASSICHEALS_OPTIONS[OPT_SHOWOVERHEAL] then
			frame:SetPoint("CENTER", parentFrame, -28, -2);
			message = "+" .. amount .. " (" .. overhealing .. ")";
		else
			frame:SetPoint("CENTER", parentFrame, -40, -2);
			
			local totalAmount = amount - overhealing;					
			if totalAmount > 0 then
				message = "+" .. totalAmount;
			else
				message = ch:GetText(ch.LKEY_FULL);
			end
		end
	else
		frame:SetPoint("CENTER", parentFrame, -41, -2);
		message = "+" .. amount;
	end

	if critical then
		UIFrameFadeIn(frame, 0.1, 0, 1);
		frame:SetFont("Fonts\\FRIZQT__.TTF", 20);			
		frame.fadeInfo.finishedFunc = function()
			UIFrameFadeOut(frame, 3, 3, 0);
		end
	else
		UIFrameFadeIn(frame, 0.3, 0, 1);
		frame:SetFont("Fonts\\FRIZQT__.TTF", 16);
		frame.fadeInfo.finishedFunc = function()
			UIFrameFadeOut(frame, 1.5, 1.5, 0);
		end
	end
	
	frame:AddMessage(message, 0, 1, 0, nil, 1);
end

function ch:GetPartyMembers()
	if partyMembers == nil then
		partyMembers = {};
		local plist = GetHomePartyInfo();
		local n = 1;
		
		if plist then
			for k,v in pairs(plist) do
				partyMembers[n] = v;
				n = n + 1;
			end
		end
		
		ch:CreatePartyFrames();
	end
	return partyMembers;
end

function ch:CreatePartyFrames()
	for k,v in pairs(ch:GetPartyMembers()) do
		if not partyFrames[k] then
			partyFrames[k] = CreateFrame("MessageFrame", "partyFrame1", UIParent);
			partyFrames[k]:SetWidth(160);
			partyFrames[k]:SetHeight(20);
			partyFrames[k]:SetPoint("CENTER", _G["PartyMemberFrame" .. k], -41, -2);
			partyFrames[k]:Show();
		end
	end
end

function ch:GetAffectedPartyMemberId(memberName)
	for playerId, playerName in ipairs(ch:GetPartyMembers()) do
		if playerName == memberName then
			return playerId;
		end
	end

	return nil;
end

function ch:Println(str)
	if str ~= nil then
		print("[ClassicHeals] " .. str);
	end
end